/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     src/proto/grpc/lookup/v1/rls.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "src/proto/grpc/lookup/v1/rls.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const grpc_lookup_v1_RouteLookupRequest_submsgs[1] = {
  &grpc_lookup_v1_RouteLookupRequest_KeyMapEntry_msginit,
};

static const upb_msglayout_field grpc_lookup_v1_RouteLookupRequest__fields[4] = {
  {3, UPB_SIZE(4, 8), 0, 0, 9, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(20, 40), 0, 0, 11, _UPB_MODE_MAP},
  {5, UPB_SIZE(0, 0), 0, 0, 14, _UPB_MODE_SCALAR},
  {6, UPB_SIZE(12, 24), 0, 0, 9, _UPB_MODE_SCALAR},
};

const upb_msglayout grpc_lookup_v1_RouteLookupRequest_msginit = {
  &grpc_lookup_v1_RouteLookupRequest_submsgs[0],
  &grpc_lookup_v1_RouteLookupRequest__fields[0],
  UPB_SIZE(24, 48), 4, false, 0, 255,
};

static const upb_msglayout_field grpc_lookup_v1_RouteLookupRequest_KeyMapEntry__fields[2] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 16), 0, 0, 9, _UPB_MODE_SCALAR},
};

const upb_msglayout grpc_lookup_v1_RouteLookupRequest_KeyMapEntry_msginit = {
  NULL,
  &grpc_lookup_v1_RouteLookupRequest_KeyMapEntry__fields[0],
  UPB_SIZE(16, 32), 2, false, 2, 255,
};

static const upb_msglayout_field grpc_lookup_v1_RouteLookupResponse__fields[2] = {
  {2, UPB_SIZE(0, 0), 0, 0, 9, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(8, 16), 0, 0, 9, _UPB_MODE_ARRAY},
};

const upb_msglayout grpc_lookup_v1_RouteLookupResponse_msginit = {
  NULL,
  &grpc_lookup_v1_RouteLookupResponse__fields[0],
  UPB_SIZE(16, 32), 2, false, 0, 255,
};

#include "upb/port_undef.inc"

