/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     udpa/annotations/status.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef UDPA_ANNOTATIONS_STATUS_PROTO_UPBDEFS_H_
#define UDPA_ANNOTATIONS_STATUS_PROTO_UPBDEFS_H_

#include "upb/def.h"
#include "upb/port_def.inc"
#ifdef __cplusplus
extern "C" {
#endif

#include "upb/def.h"

#include "upb/port_def.inc"

extern upb_def_init udpa_annotations_status_proto_upbdefinit;

UPB_INLINE const upb_msgdef *udpa_annotations_StatusAnnotation_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &udpa_annotations_status_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "udpa.annotations.StatusAnnotation");
}

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port_undef.inc"

#endif  /* UDPA_ANNOTATIONS_STATUS_PROTO_UPBDEFS_H_ */
