/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/config/endpoint/v3/load_report.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "envoy/config/endpoint/v3/load_report.upb.h"
#include "envoy/config/core/v3/address.upb.h"
#include "envoy/config/core/v3/base.upb.h"
#include "google/protobuf/duration.upb.h"
#include "google/protobuf/struct.upb.h"
#include "udpa/annotations/status.upb.h"
#include "udpa/annotations/versioning.upb.h"
#include "validate/validate.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const envoy_config_endpoint_v3_UpstreamLocalityStats_submsgs[3] = {
  &envoy_config_core_v3_Locality_msginit,
  &envoy_config_endpoint_v3_EndpointLoadMetricStats_msginit,
  &envoy_config_endpoint_v3_UpstreamEndpointStats_msginit,
};

static const upb_msglayout_field envoy_config_endpoint_v3_UpstreamLocalityStats__fields[8] = {
  {1, UPB_SIZE(44, 48), 1, 0, 11, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 8), 0, 0, 4, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(16, 16), 0, 0, 4, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(24, 24), 0, 0, 4, _UPB_MODE_SCALAR},
  {5, UPB_SIZE(48, 56), 0, 1, 11, _UPB_MODE_ARRAY},
  {6, UPB_SIZE(40, 40), 0, 0, 13, _UPB_MODE_SCALAR},
  {7, UPB_SIZE(52, 64), 0, 2, 11, _UPB_MODE_ARRAY},
  {8, UPB_SIZE(32, 32), 0, 0, 4, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_endpoint_v3_UpstreamLocalityStats_msginit = {
  &envoy_config_endpoint_v3_UpstreamLocalityStats_submsgs[0],
  &envoy_config_endpoint_v3_UpstreamLocalityStats__fields[0],
  UPB_SIZE(56, 72), 8, false, 8, 255,
};

static const upb_msglayout *const envoy_config_endpoint_v3_UpstreamEndpointStats_submsgs[3] = {
  &envoy_config_core_v3_Address_msginit,
  &envoy_config_endpoint_v3_EndpointLoadMetricStats_msginit,
  &google_protobuf_Struct_msginit,
};

static const upb_msglayout_field envoy_config_endpoint_v3_UpstreamEndpointStats__fields[7] = {
  {1, UPB_SIZE(40, 40), 1, 0, 11, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 8), 0, 0, 4, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(16, 16), 0, 0, 4, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(24, 24), 0, 0, 4, _UPB_MODE_SCALAR},
  {5, UPB_SIZE(48, 56), 0, 1, 11, _UPB_MODE_ARRAY},
  {6, UPB_SIZE(44, 48), 2, 2, 11, _UPB_MODE_SCALAR},
  {7, UPB_SIZE(32, 32), 0, 0, 4, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_endpoint_v3_UpstreamEndpointStats_msginit = {
  &envoy_config_endpoint_v3_UpstreamEndpointStats_submsgs[0],
  &envoy_config_endpoint_v3_UpstreamEndpointStats__fields[0],
  UPB_SIZE(56, 64), 7, false, 7, 255,
};

static const upb_msglayout_field envoy_config_endpoint_v3_EndpointLoadMetricStats__fields[3] = {
  {1, UPB_SIZE(16, 16), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(0, 0), 0, 0, 4, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(8, 8), 0, 0, 1, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_endpoint_v3_EndpointLoadMetricStats_msginit = {
  NULL,
  &envoy_config_endpoint_v3_EndpointLoadMetricStats__fields[0],
  UPB_SIZE(24, 32), 3, false, 3, 255,
};

static const upb_msglayout *const envoy_config_endpoint_v3_ClusterStats_submsgs[3] = {
  &envoy_config_endpoint_v3_ClusterStats_DroppedRequests_msginit,
  &envoy_config_endpoint_v3_UpstreamLocalityStats_msginit,
  &google_protobuf_Duration_msginit,
};

static const upb_msglayout_field envoy_config_endpoint_v3_ClusterStats__fields[6] = {
  {1, UPB_SIZE(16, 16), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(36, 56), 0, 1, 11, _UPB_MODE_ARRAY},
  {3, UPB_SIZE(8, 8), 0, 0, 4, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(32, 48), 1, 2, 11, _UPB_MODE_SCALAR},
  {5, UPB_SIZE(40, 64), 0, 0, 11, _UPB_MODE_ARRAY},
  {6, UPB_SIZE(24, 32), 0, 0, 9, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_endpoint_v3_ClusterStats_msginit = {
  &envoy_config_endpoint_v3_ClusterStats_submsgs[0],
  &envoy_config_endpoint_v3_ClusterStats__fields[0],
  UPB_SIZE(48, 80), 6, false, 6, 255,
};

static const upb_msglayout_field envoy_config_endpoint_v3_ClusterStats_DroppedRequests__fields[2] = {
  {1, UPB_SIZE(8, 8), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(0, 0), 0, 0, 4, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_endpoint_v3_ClusterStats_DroppedRequests_msginit = {
  NULL,
  &envoy_config_endpoint_v3_ClusterStats_DroppedRequests__fields[0],
  UPB_SIZE(16, 32), 2, false, 2, 255,
};

#include "upb/port_undef.inc"

