/*
 * Copyright 2015 gRPC authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * WARNING: Auto-generated code.
 *
 * To make changes to this file, change
 * tools/codegen/core/gen_static_metadata.py, and then re-run it.
 *
 * See metadata.h for an explanation of the interface here, and metadata.cc for
 * an explanation of what's going on.
 */

#include <grpc/support/port_platform.h>

#include "src/core/lib/slice/static_slice.h"

namespace grpc_core {
const uint8_t g_static_metadata_bytes[] = {
    58,  112, 97,  116, 104, 58,  109, 101, 116, 104, 111, 100, 58,  115, 116,
    97,  116, 117, 115, 58,  97,  117, 116, 104, 111, 114, 105, 116, 121, 58,
    115, 99,  104, 101, 109, 101, 103, 114, 112, 99,  45,  109, 101, 115, 115,
    97,  103, 101, 103, 114, 112, 99,  45,  115, 116, 97,  116, 117, 115, 103,
    114, 112, 99,  45,  112, 97,  121, 108, 111, 97,  100, 45,  98,  105, 110,
    103, 114, 112, 99,  45,  101, 110, 99,  111, 100, 105, 110, 103, 103, 114,
    112, 99,  45,  97,  99,  99,  101, 112, 116, 45,  101, 110, 99,  111, 100,
    105, 110, 103, 103, 114, 112, 99,  45,  115, 101, 114, 118, 101, 114, 45,
    115, 116, 97,  116, 115, 45,  98,  105, 110, 103, 114, 112, 99,  45,  116,
    97,  103, 115, 45,  98,  105, 110, 103, 114, 112, 99,  45,  116, 114, 97,
    99,  101, 45,  98,  105, 110, 99,  111, 110, 116, 101, 110, 116, 45,  116,
    121, 112, 101, 99,  111, 110, 116, 101, 110, 116, 45,  101, 110, 99,  111,
    100, 105, 110, 103, 97,  99,  99,  101, 112, 116, 45,  101, 110, 99,  111,
    100, 105, 110, 103, 103, 114, 112, 99,  45,  105, 110, 116, 101, 114, 110,
    97,  108, 45,  101, 110, 99,  111, 100, 105, 110, 103, 45,  114, 101, 113,
    117, 101, 115, 116, 103, 114, 112, 99,  45,  105, 110, 116, 101, 114, 110,
    97,  108, 45,  115, 116, 114, 101, 97,  109, 45,  101, 110, 99,  111, 100,
    105, 110, 103, 45,  114, 101, 113, 117, 101, 115, 116, 117, 115, 101, 114,
    45,  97,  103, 101, 110, 116, 104, 111, 115, 116, 103, 114, 112, 99,  45,
    112, 114, 101, 118, 105, 111, 117, 115, 45,  114, 112, 99,  45,  97,  116,
    116, 101, 109, 112, 116, 115, 103, 114, 112, 99,  45,  114, 101, 116, 114,
    121, 45,  112, 117, 115, 104, 98,  97,  99,  107, 45,  109, 115, 120, 45,
    101, 110, 100, 112, 111, 105, 110, 116, 45,  108, 111, 97,  100, 45,  109,
    101, 116, 114, 105, 99,  115, 45,  98,  105, 110, 103, 114, 112, 99,  45,
    116, 105, 109, 101, 111, 117, 116, 49,  50,  51,  52,  103, 114, 112, 99,
    46,  119, 97,  105, 116, 95,  102, 111, 114, 95,  114, 101, 97,  100, 121,
    103, 114, 112, 99,  46,  116, 105, 109, 101, 111, 117, 116, 103, 114, 112,
    99,  46,  109, 97,  120, 95,  114, 101, 113, 117, 101, 115, 116, 95,  109,
    101, 115, 115, 97,  103, 101, 95,  98,  121, 116, 101, 115, 103, 114, 112,
    99,  46,  109, 97,  120, 95,  114, 101, 115, 112, 111, 110, 115, 101, 95,
    109, 101, 115, 115, 97,  103, 101, 95,  98,  121, 116, 101, 115, 47,  103,
    114, 112, 99,  46,  108, 98,  46,  118, 49,  46,  76,  111, 97,  100, 66,
    97,  108, 97,  110, 99,  101, 114, 47,  66,  97,  108, 97,  110, 99,  101,
    76,  111, 97,  100, 47,  101, 110, 118, 111, 121, 46,  115, 101, 114, 118,
    105, 99,  101, 46,  108, 111, 97,  100, 95,  115, 116, 97,  116, 115, 46,
    118, 50,  46,  76,  111, 97,  100, 82,  101, 112, 111, 114, 116, 105, 110,
    103, 83,  101, 114, 118, 105, 99,  101, 47,  83,  116, 114, 101, 97,  109,
    76,  111, 97,  100, 83,  116, 97,  116, 115, 47,  101, 110, 118, 111, 121,
    46,  115, 101, 114, 118, 105, 99,  101, 46,  108, 111, 97,  100, 95,  115,
    116, 97,  116, 115, 46,  118, 51,  46,  76,  111, 97,  100, 82,  101, 112,
    111, 114, 116, 105, 110, 103, 83,  101, 114, 118, 105, 99,  101, 47,  83,
    116, 114, 101, 97,  109, 76,  111, 97,  100, 83,  116, 97,  116, 115, 47,
    103, 114, 112, 99,  46,  104, 101, 97,  108, 116, 104, 46,  118, 49,  46,
    72,  101, 97,  108, 116, 104, 47,  87,  97,  116, 99,  104, 47,  101, 110,
    118, 111, 121, 46,  115, 101, 114, 118, 105, 99,  101, 46,  100, 105, 115,
    99,  111, 118, 101, 114, 121, 46,  118, 50,  46,  65,  103, 103, 114, 101,
    103, 97,  116, 101, 100, 68,  105, 115, 99,  111, 118, 101, 114, 121, 83,
    101, 114, 118, 105, 99,  101, 47,  83,  116, 114, 101, 97,  109, 65,  103,
    103, 114, 101, 103, 97,  116, 101, 100, 82,  101, 115, 111, 117, 114, 99,
    101, 115, 47,  101, 110, 118, 111, 121, 46,  115, 101, 114, 118, 105, 99,
    101, 46,  100, 105, 115, 99,  111, 118, 101, 114, 121, 46,  118, 51,  46,
    65,  103, 103, 114, 101, 103, 97,  116, 101, 100, 68,  105, 115, 99,  111,
    118, 101, 114, 121, 83,  101, 114, 118, 105, 99,  101, 47,  83,  116, 114,
    101, 97,  109, 65,  103, 103, 114, 101, 103, 97,  116, 101, 100, 82,  101,
    115, 111, 117, 114, 99,  101, 115, 100, 101, 102, 108, 97,  116, 101, 103,
    122, 105, 112, 115, 116, 114, 101, 97,  109, 47,  103, 122, 105, 112, 116,
    101, 116, 114, 97,  105, 108, 101, 114, 115, 71,  69,  84,  80,  79,  83,
    84,  47,  47,  105, 110, 100, 101, 120, 46,  104, 116, 109, 108, 104, 116,
    116, 112, 104, 116, 116, 112, 115, 50,  48,  48,  50,  48,  52,  50,  48,
    54,  51,  48,  52,  52,  48,  48,  52,  48,  52,  53,  48,  48,  97,  99,
    99,  101, 112, 116, 45,  99,  104, 97,  114, 115, 101, 116, 103, 122, 105,
    112, 44,  32,  100, 101, 102, 108, 97,  116, 101, 97,  99,  99,  101, 112,
    116, 45,  108, 97,  110, 103, 117, 97,  103, 101, 97,  99,  99,  101, 112,
    116, 45,  114, 97,  110, 103, 101, 115, 97,  99,  99,  101, 112, 116, 97,
    99,  99,  101, 115, 115, 45,  99,  111, 110, 116, 114, 111, 108, 45,  97,
    108, 108, 111, 119, 45,  111, 114, 105, 103, 105, 110, 97,  103, 101, 97,
    108, 108, 111, 119, 97,  117, 116, 104, 111, 114, 105, 122, 97,  116, 105,
    111, 110, 99,  97,  99,  104, 101, 45,  99,  111, 110, 116, 114, 111, 108,
    99,  111, 110, 116, 101, 110, 116, 45,  100, 105, 115, 112, 111, 115, 105,
    116, 105, 111, 110, 99,  111, 110, 116, 101, 110, 116, 45,  108, 97,  110,
    103, 117, 97,  103, 101, 99,  111, 110, 116, 101, 110, 116, 45,  108, 101,
    110, 103, 116, 104, 99,  111, 110, 116, 101, 110, 116, 45,  108, 111, 99,
    97,  116, 105, 111, 110, 99,  111, 110, 116, 101, 110, 116, 45,  114, 97,
    110, 103, 101, 99,  111, 111, 107, 105, 101, 100, 97,  116, 101, 101, 116,
    97,  103, 101, 120, 112, 101, 99,  116, 101, 120, 112, 105, 114, 101, 115,
    102, 114, 111, 109, 105, 102, 45,  109, 97,  116, 99,  104, 105, 102, 45,
    109, 111, 100, 105, 102, 105, 101, 100, 45,  115, 105, 110, 99,  101, 105,
    102, 45,  110, 111, 110, 101, 45,  109, 97,  116, 99,  104, 105, 102, 45,
    114, 97,  110, 103, 101, 105, 102, 45,  117, 110, 109, 111, 100, 105, 102,
    105, 101, 100, 45,  115, 105, 110, 99,  101, 108, 97,  115, 116, 45,  109,
    111, 100, 105, 102, 105, 101, 100, 108, 105, 110, 107, 108, 111, 99,  97,
    116, 105, 111, 110, 109, 97,  120, 45,  102, 111, 114, 119, 97,  114, 100,
    115, 112, 114, 111, 120, 121, 45,  97,  117, 116, 104, 101, 110, 116, 105,
    99,  97,  116, 101, 112, 114, 111, 120, 121, 45,  97,  117, 116, 104, 111,
    114, 105, 122, 97,  116, 105, 111, 110, 114, 97,  110, 103, 101, 114, 101,
    102, 101, 114, 101, 114, 114, 101, 102, 114, 101, 115, 104, 114, 101, 116,
    114, 121, 45,  97,  102, 116, 101, 114, 115, 101, 114, 118, 101, 114, 115,
    101, 116, 45,  99,  111, 111, 107, 105, 101, 115, 116, 114, 105, 99,  116,
    45,  116, 114, 97,  110, 115, 112, 111, 114, 116, 45,  115, 101, 99,  117,
    114, 105, 116, 121, 116, 114, 97,  110, 115, 102, 101, 114, 45,  101, 110,
    99,  111, 100, 105, 110, 103, 118, 97,  114, 121, 118, 105, 97,  119, 119,
    119, 45,  97,  117, 116, 104, 101, 110, 116, 105, 99,  97,  116, 101, 48,
    105, 100, 101, 110, 116, 105, 116, 121, 97,  112, 112, 108, 105, 99,  97,
    116, 105, 111, 110, 47,  103, 114, 112, 99,  103, 114, 112, 99,  80,  85,
    84,  108, 98,  45,  99,  111, 115, 116, 45,  98,  105, 110, 105, 100, 101,
    110, 116, 105, 116, 121, 44,  100, 101, 102, 108, 97,  116, 101, 105, 100,
    101, 110, 116, 105, 116, 121, 44,  103, 122, 105, 112, 100, 101, 102, 108,
    97,  116, 101, 44,  103, 122, 105, 112, 105, 100, 101, 110, 116, 105, 116,
    121, 44,  100, 101, 102, 108, 97,  116, 101, 44,  103, 122, 105, 112};

grpc_slice_refcount grpc_core::StaticSliceRefcount::kStaticSubRefcount;

StaticSliceRefcount g_static_metadata_slice_refcounts[GRPC_STATIC_MDSTR_COUNT] =
    {

        StaticSliceRefcount(0),   StaticSliceRefcount(1),
        StaticSliceRefcount(2),   StaticSliceRefcount(3),
        StaticSliceRefcount(4),   StaticSliceRefcount(5),
        StaticSliceRefcount(6),   StaticSliceRefcount(7),
        StaticSliceRefcount(8),   StaticSliceRefcount(9),
        StaticSliceRefcount(10),  StaticSliceRefcount(11),
        StaticSliceRefcount(12),  StaticSliceRefcount(13),
        StaticSliceRefcount(14),  StaticSliceRefcount(15),
        StaticSliceRefcount(16),  StaticSliceRefcount(17),
        StaticSliceRefcount(18),  StaticSliceRefcount(19),
        StaticSliceRefcount(20),  StaticSliceRefcount(21),
        StaticSliceRefcount(22),  StaticSliceRefcount(23),
        StaticSliceRefcount(24),  StaticSliceRefcount(25),
        StaticSliceRefcount(26),  StaticSliceRefcount(27),
        StaticSliceRefcount(28),  StaticSliceRefcount(29),
        StaticSliceRefcount(30),  StaticSliceRefcount(31),
        StaticSliceRefcount(32),  StaticSliceRefcount(33),
        StaticSliceRefcount(34),  StaticSliceRefcount(35),
        StaticSliceRefcount(36),  StaticSliceRefcount(37),
        StaticSliceRefcount(38),  StaticSliceRefcount(39),
        StaticSliceRefcount(40),  StaticSliceRefcount(41),
        StaticSliceRefcount(42),  StaticSliceRefcount(43),
        StaticSliceRefcount(44),  StaticSliceRefcount(45),
        StaticSliceRefcount(46),  StaticSliceRefcount(47),
        StaticSliceRefcount(48),  StaticSliceRefcount(49),
        StaticSliceRefcount(50),  StaticSliceRefcount(51),
        StaticSliceRefcount(52),  StaticSliceRefcount(53),
        StaticSliceRefcount(54),  StaticSliceRefcount(55),
        StaticSliceRefcount(56),  StaticSliceRefcount(57),
        StaticSliceRefcount(58),  StaticSliceRefcount(59),
        StaticSliceRefcount(60),  StaticSliceRefcount(61),
        StaticSliceRefcount(62),  StaticSliceRefcount(63),
        StaticSliceRefcount(64),  StaticSliceRefcount(65),
        StaticSliceRefcount(66),  StaticSliceRefcount(67),
        StaticSliceRefcount(68),  StaticSliceRefcount(69),
        StaticSliceRefcount(70),  StaticSliceRefcount(71),
        StaticSliceRefcount(72),  StaticSliceRefcount(73),
        StaticSliceRefcount(74),  StaticSliceRefcount(75),
        StaticSliceRefcount(76),  StaticSliceRefcount(77),
        StaticSliceRefcount(78),  StaticSliceRefcount(79),
        StaticSliceRefcount(80),  StaticSliceRefcount(81),
        StaticSliceRefcount(82),  StaticSliceRefcount(83),
        StaticSliceRefcount(84),  StaticSliceRefcount(85),
        StaticSliceRefcount(86),  StaticSliceRefcount(87),
        StaticSliceRefcount(88),  StaticSliceRefcount(89),
        StaticSliceRefcount(90),  StaticSliceRefcount(91),
        StaticSliceRefcount(92),  StaticSliceRefcount(93),
        StaticSliceRefcount(94),  StaticSliceRefcount(95),
        StaticSliceRefcount(96),  StaticSliceRefcount(97),
        StaticSliceRefcount(98),  StaticSliceRefcount(99),
        StaticSliceRefcount(100), StaticSliceRefcount(101),
        StaticSliceRefcount(102), StaticSliceRefcount(103),
        StaticSliceRefcount(104), StaticSliceRefcount(105),
        StaticSliceRefcount(106), StaticSliceRefcount(107),
        StaticSliceRefcount(108), StaticSliceRefcount(109),
};

const StaticMetadataSlice
    g_static_metadata_slice_table[GRPC_STATIC_MDSTR_COUNT] = {

        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[0].base, 5,
            g_static_metadata_bytes + 0),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[1].base, 7,
            g_static_metadata_bytes + 5),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[2].base, 7,
            g_static_metadata_bytes + 12),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[3].base, 10,
            g_static_metadata_bytes + 19),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[4].base, 7,
            g_static_metadata_bytes + 29),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[5].base, 12,
            g_static_metadata_bytes + 36),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[6].base, 11,
            g_static_metadata_bytes + 48),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[7].base, 16,
            g_static_metadata_bytes + 59),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[8].base, 13,
            g_static_metadata_bytes + 75),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[9].base, 20,
            g_static_metadata_bytes + 88),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[10].base, 21,
            g_static_metadata_bytes + 108),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[11].base, 13,
            g_static_metadata_bytes + 129),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[12].base, 14,
            g_static_metadata_bytes + 142),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[13].base, 12,
            g_static_metadata_bytes + 156),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[14].base, 16,
            g_static_metadata_bytes + 168),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[15].base, 15,
            g_static_metadata_bytes + 184),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[16].base, 30,
            g_static_metadata_bytes + 199),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[17].base, 37,
            g_static_metadata_bytes + 229),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[18].base, 10,
            g_static_metadata_bytes + 266),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[19].base, 4,
            g_static_metadata_bytes + 276),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[20].base, 26,
            g_static_metadata_bytes + 280),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[21].base, 22,
            g_static_metadata_bytes + 306),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[22].base, 27,
            g_static_metadata_bytes + 328),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[23].base, 12,
            g_static_metadata_bytes + 355),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[24].base, 1,
            g_static_metadata_bytes + 367),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[25].base, 1,
            g_static_metadata_bytes + 368),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[26].base, 1,
            g_static_metadata_bytes + 369),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[27].base, 1,
            g_static_metadata_bytes + 370),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[28].base, 0,
            g_static_metadata_bytes + 371),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[29].base, 19,
            g_static_metadata_bytes + 371),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[30].base, 12,
            g_static_metadata_bytes + 390),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[31].base, 30,
            g_static_metadata_bytes + 402),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[32].base, 31,
            g_static_metadata_bytes + 432),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[33].base, 36,
            g_static_metadata_bytes + 463),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[34].base, 65,
            g_static_metadata_bytes + 499),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[35].base, 65,
            g_static_metadata_bytes + 564),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[36].base, 28,
            g_static_metadata_bytes + 629),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[37].base, 80,
            g_static_metadata_bytes + 657),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[38].base, 80,
            g_static_metadata_bytes + 737),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[39].base, 7,
            g_static_metadata_bytes + 817),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[40].base, 4,
            g_static_metadata_bytes + 824),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[41].base, 11,
            g_static_metadata_bytes + 828),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[42].base, 2,
            g_static_metadata_bytes + 839),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[43].base, 8,
            g_static_metadata_bytes + 841),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[44].base, 3,
            g_static_metadata_bytes + 849),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[45].base, 4,
            g_static_metadata_bytes + 852),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[46].base, 1,
            g_static_metadata_bytes + 856),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[47].base, 11,
            g_static_metadata_bytes + 857),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[48].base, 4,
            g_static_metadata_bytes + 868),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[49].base, 5,
            g_static_metadata_bytes + 872),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[50].base, 3,
            g_static_metadata_bytes + 877),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[51].base, 3,
            g_static_metadata_bytes + 880),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[52].base, 3,
            g_static_metadata_bytes + 883),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[53].base, 3,
            g_static_metadata_bytes + 886),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[54].base, 3,
            g_static_metadata_bytes + 889),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[55].base, 3,
            g_static_metadata_bytes + 892),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[56].base, 3,
            g_static_metadata_bytes + 895),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[57].base, 14,
            g_static_metadata_bytes + 898),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[58].base, 13,
            g_static_metadata_bytes + 912),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[59].base, 15,
            g_static_metadata_bytes + 925),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[60].base, 13,
            g_static_metadata_bytes + 940),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[61].base, 6,
            g_static_metadata_bytes + 953),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[62].base, 27,
            g_static_metadata_bytes + 959),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[63].base, 3,
            g_static_metadata_bytes + 986),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[64].base, 5,
            g_static_metadata_bytes + 989),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[65].base, 13,
            g_static_metadata_bytes + 994),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[66].base, 13,
            g_static_metadata_bytes + 1007),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[67].base, 19,
            g_static_metadata_bytes + 1020),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[68].base, 16,
            g_static_metadata_bytes + 1039),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[69].base, 14,
            g_static_metadata_bytes + 1055),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[70].base, 16,
            g_static_metadata_bytes + 1069),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[71].base, 13,
            g_static_metadata_bytes + 1085),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[72].base, 6,
            g_static_metadata_bytes + 1098),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[73].base, 4,
            g_static_metadata_bytes + 1104),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[74].base, 4,
            g_static_metadata_bytes + 1108),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[75].base, 6,
            g_static_metadata_bytes + 1112),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[76].base, 7,
            g_static_metadata_bytes + 1118),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[77].base, 4,
            g_static_metadata_bytes + 1125),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[78].base, 8,
            g_static_metadata_bytes + 1129),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[79].base, 17,
            g_static_metadata_bytes + 1137),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[80].base, 13,
            g_static_metadata_bytes + 1154),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[81].base, 8,
            g_static_metadata_bytes + 1167),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[82].base, 19,
            g_static_metadata_bytes + 1175),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[83].base, 13,
            g_static_metadata_bytes + 1194),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[84].base, 4,
            g_static_metadata_bytes + 1207),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[85].base, 8,
            g_static_metadata_bytes + 1211),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[86].base, 12,
            g_static_metadata_bytes + 1219),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[87].base, 18,
            g_static_metadata_bytes + 1231),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[88].base, 19,
            g_static_metadata_bytes + 1249),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[89].base, 5,
            g_static_metadata_bytes + 1268),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[90].base, 7,
            g_static_metadata_bytes + 1273),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[91].base, 7,
            g_static_metadata_bytes + 1280),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[92].base, 11,
            g_static_metadata_bytes + 1287),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[93].base, 6,
            g_static_metadata_bytes + 1298),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[94].base, 10,
            g_static_metadata_bytes + 1304),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[95].base, 25,
            g_static_metadata_bytes + 1314),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[96].base, 17,
            g_static_metadata_bytes + 1339),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[97].base, 4,
            g_static_metadata_bytes + 1356),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[98].base, 3,
            g_static_metadata_bytes + 1360),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[99].base, 16,
            g_static_metadata_bytes + 1363),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[100].base, 1,
            g_static_metadata_bytes + 1379),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[101].base, 8,
            g_static_metadata_bytes + 1380),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[102].base, 16,
            g_static_metadata_bytes + 1388),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[103].base, 4,
            g_static_metadata_bytes + 1404),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[104].base, 3,
            g_static_metadata_bytes + 1408),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[105].base, 11,
            g_static_metadata_bytes + 1411),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[106].base, 16,
            g_static_metadata_bytes + 1422),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[107].base, 13,
            g_static_metadata_bytes + 1438),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[108].base, 12,
            g_static_metadata_bytes + 1451),
        grpc_core::StaticMetadataSlice(
            &g_static_metadata_slice_refcounts[109].base, 21,
            g_static_metadata_bytes + 1463),
};
}  // namespace grpc_core
