/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     google/protobuf/wrappers.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef GOOGLE_PROTOBUF_WRAPPERS_PROTO_UPBDEFS_H_
#define GOOGLE_PROTOBUF_WRAPPERS_PROTO_UPBDEFS_H_

#include "upb/def.h"
#include "upb/port_def.inc"
#ifdef __cplusplus
extern "C" {
#endif

#include "upb/def.h"

#include "upb/port_def.inc"

extern upb_def_init google_protobuf_wrappers_proto_upbdefinit;

UPB_INLINE const upb_msgdef *google_protobuf_DoubleValue_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &google_protobuf_wrappers_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "google.protobuf.DoubleValue");
}

UPB_INLINE const upb_msgdef *google_protobuf_FloatValue_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &google_protobuf_wrappers_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "google.protobuf.FloatValue");
}

UPB_INLINE const upb_msgdef *google_protobuf_Int64Value_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &google_protobuf_wrappers_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "google.protobuf.Int64Value");
}

UPB_INLINE const upb_msgdef *google_protobuf_UInt64Value_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &google_protobuf_wrappers_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "google.protobuf.UInt64Value");
}

UPB_INLINE const upb_msgdef *google_protobuf_Int32Value_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &google_protobuf_wrappers_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "google.protobuf.Int32Value");
}

UPB_INLINE const upb_msgdef *google_protobuf_UInt32Value_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &google_protobuf_wrappers_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "google.protobuf.UInt32Value");
}

UPB_INLINE const upb_msgdef *google_protobuf_BoolValue_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &google_protobuf_wrappers_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "google.protobuf.BoolValue");
}

UPB_INLINE const upb_msgdef *google_protobuf_StringValue_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &google_protobuf_wrappers_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "google.protobuf.StringValue");
}

UPB_INLINE const upb_msgdef *google_protobuf_BytesValue_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &google_protobuf_wrappers_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "google.protobuf.BytesValue");
}

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port_undef.inc"

#endif  /* GOOGLE_PROTOBUF_WRAPPERS_PROTO_UPBDEFS_H_ */
