/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/extensions/transport_sockets/tls/v3/common.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef ENVOY_EXTENSIONS_TRANSPORT_SOCKETS_TLS_V3_COMMON_PROTO_UPBDEFS_H_
#define ENVOY_EXTENSIONS_TRANSPORT_SOCKETS_TLS_V3_COMMON_PROTO_UPBDEFS_H_

#include "upb/def.h"
#include "upb/port_def.inc"
#ifdef __cplusplus
extern "C" {
#endif

#include "upb/def.h"

#include "upb/port_def.inc"

extern upb_def_init envoy_extensions_transport_sockets_tls_v3_common_proto_upbdefinit;

UPB_INLINE const upb_msgdef *envoy_extensions_transport_sockets_tls_v3_TlsParameters_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_extensions_transport_sockets_tls_v3_common_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.extensions.transport_sockets.tls.v3.TlsParameters");
}

UPB_INLINE const upb_msgdef *envoy_extensions_transport_sockets_tls_v3_PrivateKeyProvider_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_extensions_transport_sockets_tls_v3_common_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.extensions.transport_sockets.tls.v3.PrivateKeyProvider");
}

UPB_INLINE const upb_msgdef *envoy_extensions_transport_sockets_tls_v3_TlsCertificate_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_extensions_transport_sockets_tls_v3_common_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.extensions.transport_sockets.tls.v3.TlsCertificate");
}

UPB_INLINE const upb_msgdef *envoy_extensions_transport_sockets_tls_v3_TlsSessionTicketKeys_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_extensions_transport_sockets_tls_v3_common_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.extensions.transport_sockets.tls.v3.TlsSessionTicketKeys");
}

UPB_INLINE const upb_msgdef *envoy_extensions_transport_sockets_tls_v3_CertificateProviderPluginInstance_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_extensions_transport_sockets_tls_v3_common_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.extensions.transport_sockets.tls.v3.CertificateProviderPluginInstance");
}

UPB_INLINE const upb_msgdef *envoy_extensions_transport_sockets_tls_v3_CertificateValidationContext_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &envoy_extensions_transport_sockets_tls_v3_common_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "envoy.extensions.transport_sockets.tls.v3.CertificateValidationContext");
}

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port_undef.inc"

#endif  /* ENVOY_EXTENSIONS_TRANSPORT_SOCKETS_TLS_V3_COMMON_PROTO_UPBDEFS_H_ */
