/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     xds/annotations/v3/status.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef XDS_ANNOTATIONS_V3_STATUS_PROTO_UPBDEFS_H_
#define XDS_ANNOTATIONS_V3_STATUS_PROTO_UPBDEFS_H_

#include "upb/def.h"
#include "upb/port_def.inc"
#ifdef __cplusplus
extern "C" {
#endif

#include "upb/def.h"

#include "upb/port_def.inc"

extern upb_def_init xds_annotations_v3_status_proto_upbdefinit;

UPB_INLINE const upb_msgdef *xds_annotations_v3_FileStatusAnnotation_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &xds_annotations_v3_status_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "xds.annotations.v3.FileStatusAnnotation");
}

UPB_INLINE const upb_msgdef *xds_annotations_v3_MessageStatusAnnotation_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &xds_annotations_v3_status_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "xds.annotations.v3.MessageStatusAnnotation");
}

UPB_INLINE const upb_msgdef *xds_annotations_v3_FieldStatusAnnotation_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &xds_annotations_v3_status_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "xds.annotations.v3.FieldStatusAnnotation");
}

UPB_INLINE const upb_msgdef *xds_annotations_v3_StatusAnnotation_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &xds_annotations_v3_status_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "xds.annotations.v3.StatusAnnotation");
}

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port_undef.inc"

#endif  /* XDS_ANNOTATIONS_V3_STATUS_PROTO_UPBDEFS_H_ */
