/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/config/metrics/v3/stats.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "envoy/config/metrics/v3/stats.upb.h"
#include "envoy/config/core/v3/address.upb.h"
#include "envoy/type/matcher/v3/string.upb.h"
#include "google/protobuf/any.upb.h"
#include "google/protobuf/wrappers.upb.h"
#include "udpa/annotations/status.upb.h"
#include "udpa/annotations/versioning.upb.h"
#include "validate/validate.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout *const envoy_config_metrics_v3_StatsSink_submsgs[1] = {
  &google_protobuf_Any_msginit,
};

static const upb_msglayout_field envoy_config_metrics_v3_StatsSink__fields[2] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(8, 16), UPB_SIZE(-13, -25), 0, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_metrics_v3_StatsSink_msginit = {
  &envoy_config_metrics_v3_StatsSink_submsgs[0],
  &envoy_config_metrics_v3_StatsSink__fields[0],
  UPB_SIZE(16, 32), 2, false, 1, 255,
};

static const upb_msglayout *const envoy_config_metrics_v3_StatsConfig_submsgs[4] = {
  &envoy_config_metrics_v3_HistogramBucketSettings_msginit,
  &envoy_config_metrics_v3_StatsMatcher_msginit,
  &envoy_config_metrics_v3_TagSpecifier_msginit,
  &google_protobuf_BoolValue_msginit,
};

static const upb_msglayout_field envoy_config_metrics_v3_StatsConfig__fields[4] = {
  {1, UPB_SIZE(12, 24), 0, 2, 11, _UPB_MODE_ARRAY},
  {2, UPB_SIZE(4, 8), 1, 3, 11, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(8, 16), 2, 1, 11, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(16, 32), 0, 0, 11, _UPB_MODE_ARRAY},
};

const upb_msglayout envoy_config_metrics_v3_StatsConfig_msginit = {
  &envoy_config_metrics_v3_StatsConfig_submsgs[0],
  &envoy_config_metrics_v3_StatsConfig__fields[0],
  UPB_SIZE(24, 40), 4, false, 4, 255,
};

static const upb_msglayout *const envoy_config_metrics_v3_StatsMatcher_submsgs[1] = {
  &envoy_type_matcher_v3_ListStringMatcher_msginit,
};

static const upb_msglayout_field envoy_config_metrics_v3_StatsMatcher__fields[3] = {
  {1, UPB_SIZE(0, 0), UPB_SIZE(-5, -9), 0, 8, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(0, 0), UPB_SIZE(-5, -9), 0, 11, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(0, 0), UPB_SIZE(-5, -9), 0, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_metrics_v3_StatsMatcher_msginit = {
  &envoy_config_metrics_v3_StatsMatcher_submsgs[0],
  &envoy_config_metrics_v3_StatsMatcher__fields[0],
  UPB_SIZE(8, 16), 3, false, 3, 255,
};

static const upb_msglayout_field envoy_config_metrics_v3_TagSpecifier__fields[3] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 16), UPB_SIZE(-17, -33), 0, 9, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(8, 16), UPB_SIZE(-17, -33), 0, 9, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_metrics_v3_TagSpecifier_msginit = {
  NULL,
  &envoy_config_metrics_v3_TagSpecifier__fields[0],
  UPB_SIZE(24, 48), 3, false, 3, 255,
};

static const upb_msglayout *const envoy_config_metrics_v3_HistogramBucketSettings_submsgs[1] = {
  &envoy_type_matcher_v3_StringMatcher_msginit,
};

static const upb_msglayout_field envoy_config_metrics_v3_HistogramBucketSettings__fields[2] = {
  {1, UPB_SIZE(4, 8), 1, 0, 11, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 16), 0, 0, 1, _UPB_MODE_ARRAY | _UPB_MODE_IS_PACKED},
};

const upb_msglayout envoy_config_metrics_v3_HistogramBucketSettings_msginit = {
  &envoy_config_metrics_v3_HistogramBucketSettings_submsgs[0],
  &envoy_config_metrics_v3_HistogramBucketSettings__fields[0],
  UPB_SIZE(16, 24), 2, false, 2, 255,
};

static const upb_msglayout *const envoy_config_metrics_v3_StatsdSink_submsgs[1] = {
  &envoy_config_core_v3_Address_msginit,
};

static const upb_msglayout_field envoy_config_metrics_v3_StatsdSink__fields[3] = {
  {1, UPB_SIZE(8, 16), UPB_SIZE(-17, -33), 0, 11, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(8, 16), UPB_SIZE(-17, -33), 0, 9, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(0, 0), 0, 0, 9, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_metrics_v3_StatsdSink_msginit = {
  &envoy_config_metrics_v3_StatsdSink_submsgs[0],
  &envoy_config_metrics_v3_StatsdSink__fields[0],
  UPB_SIZE(24, 48), 3, false, 3, 255,
};

static const upb_msglayout *const envoy_config_metrics_v3_DogStatsdSink_submsgs[2] = {
  &envoy_config_core_v3_Address_msginit,
  &google_protobuf_UInt64Value_msginit,
};

static const upb_msglayout_field envoy_config_metrics_v3_DogStatsdSink__fields[3] = {
  {1, UPB_SIZE(16, 32), UPB_SIZE(-21, -41), 0, 11, _UPB_MODE_SCALAR},
  {3, UPB_SIZE(4, 8), 0, 0, 9, _UPB_MODE_SCALAR},
  {4, UPB_SIZE(12, 24), 1, 1, 11, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_metrics_v3_DogStatsdSink_msginit = {
  &envoy_config_metrics_v3_DogStatsdSink_submsgs[0],
  &envoy_config_metrics_v3_DogStatsdSink__fields[0],
  UPB_SIZE(24, 48), 3, false, 1, 255,
};

static const upb_msglayout_field envoy_config_metrics_v3_HystrixSink__fields[1] = {
  {1, UPB_SIZE(0, 0), 0, 0, 3, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_config_metrics_v3_HystrixSink_msginit = {
  NULL,
  &envoy_config_metrics_v3_HystrixSink__fields[0],
  UPB_SIZE(8, 8), 1, false, 1, 255,
};

#include "upb/port_undef.inc"

