/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     xds/data/orca/v3/orca_load_report.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#ifndef XDS_DATA_ORCA_V3_ORCA_LOAD_REPORT_PROTO_UPBDEFS_H_
#define XDS_DATA_ORCA_V3_ORCA_LOAD_REPORT_PROTO_UPBDEFS_H_

#include "upb/def.h"
#include "upb/port_def.inc"
#ifdef __cplusplus
extern "C" {
#endif

#include "upb/def.h"

#include "upb/port_def.inc"

extern upb_def_init xds_data_orca_v3_orca_load_report_proto_upbdefinit;

UPB_INLINE const upb_msgdef *xds_data_orca_v3_OrcaLoadReport_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &xds_data_orca_v3_orca_load_report_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "xds.data.orca.v3.OrcaLoadReport");
}

UPB_INLINE const upb_msgdef *xds_data_orca_v3_OrcaLoadReport_RequestCostEntry_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &xds_data_orca_v3_orca_load_report_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "xds.data.orca.v3.OrcaLoadReport.RequestCostEntry");
}

UPB_INLINE const upb_msgdef *xds_data_orca_v3_OrcaLoadReport_UtilizationEntry_getmsgdef(upb_symtab *s) {
  _upb_symtab_loaddefinit(s, &xds_data_orca_v3_orca_load_report_proto_upbdefinit);
  return upb_symtab_lookupmsg(s, "xds.data.orca.v3.OrcaLoadReport.UtilizationEntry");
}

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port_undef.inc"

#endif  /* XDS_DATA_ORCA_V3_ORCA_LOAD_REPORT_PROTO_UPBDEFS_H_ */
