/*
 * Copyright 2015 gRPC authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * WARNING: Auto-generated code.
 *
 * To make changes to this file, change
 * tools/codegen/core/gen_static_metadata.py, and then re-run it.
 *
 * See metadata.h for an explanation of the interface here, and metadata.cc for
 * an explanation of what's going on.
 */

#include <grpc/support/port_platform.h>

#include "src/core/lib/transport/static_metadata.h"

#include "src/core/lib/slice/slice_internal.h"

namespace grpc_core {
StaticMetadata g_static_mdelem_table[GRPC_STATIC_MDELEM_COUNT] = {
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[3].base, 10,
                       g_static_metadata_bytes + 19),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   0),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[1].base, 7,
                       g_static_metadata_bytes + 5),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[44].base, 3,
                       g_static_metadata_bytes + 849),
                   1),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[1].base, 7,
                       g_static_metadata_bytes + 5),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[45].base, 4,
                       g_static_metadata_bytes + 852),
                   2),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[0].base, 5,
                       g_static_metadata_bytes + 0),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[46].base, 1,
                       g_static_metadata_bytes + 856),
                   3),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[0].base, 5,
                       g_static_metadata_bytes + 0),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[47].base, 11,
                       g_static_metadata_bytes + 857),
                   4),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[4].base, 7,
                       g_static_metadata_bytes + 29),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[48].base, 4,
                       g_static_metadata_bytes + 868),
                   5),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[4].base, 7,
                       g_static_metadata_bytes + 29),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[49].base, 5,
                       g_static_metadata_bytes + 872),
                   6),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[2].base, 7,
                       g_static_metadata_bytes + 12),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[50].base, 3,
                       g_static_metadata_bytes + 877),
                   7),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[2].base, 7,
                       g_static_metadata_bytes + 12),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[51].base, 3,
                       g_static_metadata_bytes + 880),
                   8),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[2].base, 7,
                       g_static_metadata_bytes + 12),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[52].base, 3,
                       g_static_metadata_bytes + 883),
                   9),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[2].base, 7,
                       g_static_metadata_bytes + 12),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[53].base, 3,
                       g_static_metadata_bytes + 886),
                   10),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[2].base, 7,
                       g_static_metadata_bytes + 12),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[54].base, 3,
                       g_static_metadata_bytes + 889),
                   11),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[2].base, 7,
                       g_static_metadata_bytes + 12),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[55].base, 3,
                       g_static_metadata_bytes + 892),
                   12),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[2].base, 7,
                       g_static_metadata_bytes + 12),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[56].base, 3,
                       g_static_metadata_bytes + 895),
                   13),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[57].base, 14,
                       g_static_metadata_bytes + 898),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   14),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[15].base, 15,
                       g_static_metadata_bytes + 184),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[58].base, 13,
                       g_static_metadata_bytes + 912),
                   15),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[59].base, 15,
                       g_static_metadata_bytes + 925),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   16),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[60].base, 13,
                       g_static_metadata_bytes + 940),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   17),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[61].base, 6,
                       g_static_metadata_bytes + 953),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   18),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[62].base, 27,
                       g_static_metadata_bytes + 959),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   19),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[63].base, 3,
                       g_static_metadata_bytes + 986),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   20),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[64].base, 5,
                       g_static_metadata_bytes + 989),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   21),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[65].base, 13,
                       g_static_metadata_bytes + 994),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   22),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[66].base, 13,
                       g_static_metadata_bytes + 1007),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   23),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[67].base, 19,
                       g_static_metadata_bytes + 1020),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   24),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[14].base, 16,
                       g_static_metadata_bytes + 168),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   25),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[68].base, 16,
                       g_static_metadata_bytes + 1039),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   26),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[69].base, 14,
                       g_static_metadata_bytes + 1055),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   27),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[70].base, 16,
                       g_static_metadata_bytes + 1069),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   28),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[71].base, 13,
                       g_static_metadata_bytes + 1085),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   29),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[13].base, 12,
                       g_static_metadata_bytes + 156),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   30),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[72].base, 6,
                       g_static_metadata_bytes + 1098),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   31),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[73].base, 4,
                       g_static_metadata_bytes + 1104),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   32),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[74].base, 4,
                       g_static_metadata_bytes + 1108),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   33),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[75].base, 6,
                       g_static_metadata_bytes + 1112),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   34),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[76].base, 7,
                       g_static_metadata_bytes + 1118),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   35),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[77].base, 4,
                       g_static_metadata_bytes + 1125),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   36),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[19].base, 4,
                       g_static_metadata_bytes + 276),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   37),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[78].base, 8,
                       g_static_metadata_bytes + 1129),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   38),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[79].base, 17,
                       g_static_metadata_bytes + 1137),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   39),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[80].base, 13,
                       g_static_metadata_bytes + 1154),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   40),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[81].base, 8,
                       g_static_metadata_bytes + 1167),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   41),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[82].base, 19,
                       g_static_metadata_bytes + 1175),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   42),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[83].base, 13,
                       g_static_metadata_bytes + 1194),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   43),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[84].base, 4,
                       g_static_metadata_bytes + 1207),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   44),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[85].base, 8,
                       g_static_metadata_bytes + 1211),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   45),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[86].base, 12,
                       g_static_metadata_bytes + 1219),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   46),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[87].base, 18,
                       g_static_metadata_bytes + 1231),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   47),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[88].base, 19,
                       g_static_metadata_bytes + 1249),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   48),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[89].base, 5,
                       g_static_metadata_bytes + 1268),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   49),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[90].base, 7,
                       g_static_metadata_bytes + 1273),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   50),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[91].base, 7,
                       g_static_metadata_bytes + 1280),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   51),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[92].base, 11,
                       g_static_metadata_bytes + 1287),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   52),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[93].base, 6,
                       g_static_metadata_bytes + 1298),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   53),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[94].base, 10,
                       g_static_metadata_bytes + 1304),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   54),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[95].base, 25,
                       g_static_metadata_bytes + 1314),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   55),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[96].base, 17,
                       g_static_metadata_bytes + 1339),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   56),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[18].base, 10,
                       g_static_metadata_bytes + 266),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   57),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[97].base, 4,
                       g_static_metadata_bytes + 1356),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   58),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[98].base, 3,
                       g_static_metadata_bytes + 1360),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   59),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[99].base, 16,
                       g_static_metadata_bytes + 1363),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   60),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[6].base, 11,
                       g_static_metadata_bytes + 48),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[100].base, 1,
                       g_static_metadata_bytes + 1379),
                   61),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[6].base, 11,
                       g_static_metadata_bytes + 48),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[24].base, 1,
                       g_static_metadata_bytes + 367),
                   62),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[6].base, 11,
                       g_static_metadata_bytes + 48),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[25].base, 1,
                       g_static_metadata_bytes + 368),
                   63),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[8].base, 13,
                       g_static_metadata_bytes + 75),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[101].base, 8,
                       g_static_metadata_bytes + 1380),
                   64),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[8].base, 13,
                       g_static_metadata_bytes + 75),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[40].base, 4,
                       g_static_metadata_bytes + 824),
                   65),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[8].base, 13,
                       g_static_metadata_bytes + 75),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[39].base, 7,
                       g_static_metadata_bytes + 817),
                   66),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[13].base, 12,
                       g_static_metadata_bytes + 156),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[102].base, 16,
                       g_static_metadata_bytes + 1388),
                   67),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[4].base, 7,
                       g_static_metadata_bytes + 29),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[103].base, 4,
                       g_static_metadata_bytes + 1404),
                   68),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[1].base, 7,
                       g_static_metadata_bytes + 5),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[104].base, 3,
                       g_static_metadata_bytes + 1408),
                   69),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[15].base, 15,
                       g_static_metadata_bytes + 184),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   70),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[14].base, 16,
                       g_static_metadata_bytes + 168),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[101].base, 8,
                       g_static_metadata_bytes + 1380),
                   71),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[14].base, 16,
                       g_static_metadata_bytes + 168),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[40].base, 4,
                       g_static_metadata_bytes + 824),
                   72),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[105].base, 11,
                       g_static_metadata_bytes + 1411),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[28].base, 0,
                       g_static_metadata_bytes + 371),
                   73),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[9].base, 20,
                       g_static_metadata_bytes + 88),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[101].base, 8,
                       g_static_metadata_bytes + 1380),
                   74),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[9].base, 20,
                       g_static_metadata_bytes + 88),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[39].base, 7,
                       g_static_metadata_bytes + 817),
                   75),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[9].base, 20,
                       g_static_metadata_bytes + 88),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[106].base, 16,
                       g_static_metadata_bytes + 1422),
                   76),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[9].base, 20,
                       g_static_metadata_bytes + 88),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[40].base, 4,
                       g_static_metadata_bytes + 824),
                   77),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[9].base, 20,
                       g_static_metadata_bytes + 88),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[107].base, 13,
                       g_static_metadata_bytes + 1438),
                   78),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[9].base, 20,
                       g_static_metadata_bytes + 88),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[108].base, 12,
                       g_static_metadata_bytes + 1451),
                   79),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[9].base, 20,
                       g_static_metadata_bytes + 88),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[109].base, 21,
                       g_static_metadata_bytes + 1463),
                   80),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[15].base, 15,
                       g_static_metadata_bytes + 184),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[101].base, 8,
                       g_static_metadata_bytes + 1380),
                   81),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[15].base, 15,
                       g_static_metadata_bytes + 184),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[40].base, 4,
                       g_static_metadata_bytes + 824),
                   82),
    StaticMetadata(grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[15].base, 15,
                       g_static_metadata_bytes + 184),
                   grpc_core::StaticMetadataSlice(
                       &g_static_metadata_slice_refcounts[107].base, 13,
                       g_static_metadata_bytes + 1438),
                   83),
};

/* Warning: the core static metadata currently operates under the soft
constraint that the first GRPC_CHTTP2_LAST_STATIC_ENTRY (61) entries must
contain metadata specified by the http2 hpack standard. The CHTTP2 transport
reads the core metadata with this assumption in mind. If the order of the core
static metadata is to be changed, then the CHTTP2 transport must be changed as
well to stop relying on the core metadata. */

grpc_mdelem g_static_mdelem_manifested[GRPC_STATIC_MDELEM_COUNT] = {
    // clang-format off
    /* GRPC_MDELEM_AUTHORITY_EMPTY: 
     ":authority": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[0].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_METHOD_GET: 
     ":method": "GET" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[1].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_METHOD_POST: 
     ":method": "POST" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[2].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_PATH_SLASH: 
     ":path": "/" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[3].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_PATH_SLASH_INDEX_DOT_HTML: 
     ":path": "/index.html" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[4].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_SCHEME_HTTP: 
     ":scheme": "http" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[5].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_SCHEME_HTTPS: 
     ":scheme": "https" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[6].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_STATUS_200: 
     ":status": "200" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[7].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_STATUS_204: 
     ":status": "204" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[8].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_STATUS_206: 
     ":status": "206" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[9].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_STATUS_304: 
     ":status": "304" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[10].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_STATUS_400: 
     ":status": "400" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[11].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_STATUS_404: 
     ":status": "404" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[12].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_STATUS_500: 
     ":status": "500" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[13].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ACCEPT_CHARSET_EMPTY: 
     "accept-charset": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[14].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ACCEPT_ENCODING_GZIP_COMMA_DEFLATE: 
     "accept-encoding": "gzip, deflate" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[15].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ACCEPT_LANGUAGE_EMPTY: 
     "accept-language": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[16].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ACCEPT_RANGES_EMPTY: 
     "accept-ranges": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[17].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ACCEPT_EMPTY: 
     "accept": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[18].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ACCESS_CONTROL_ALLOW_ORIGIN_EMPTY: 
     "access-control-allow-origin": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[19].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_AGE_EMPTY: 
     "age": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[20].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ALLOW_EMPTY: 
     "allow": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[21].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_AUTHORIZATION_EMPTY: 
     "authorization": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[22].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CACHE_CONTROL_EMPTY: 
     "cache-control": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[23].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CONTENT_DISPOSITION_EMPTY: 
     "content-disposition": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[24].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CONTENT_ENCODING_EMPTY: 
     "content-encoding": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[25].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CONTENT_LANGUAGE_EMPTY: 
     "content-language": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[26].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CONTENT_LENGTH_EMPTY: 
     "content-length": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[27].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CONTENT_LOCATION_EMPTY: 
     "content-location": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[28].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CONTENT_RANGE_EMPTY: 
     "content-range": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[29].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CONTENT_TYPE_EMPTY: 
     "content-type": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[30].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_COOKIE_EMPTY: 
     "cookie": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[31].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_DATE_EMPTY: 
     "date": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[32].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ETAG_EMPTY: 
     "etag": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[33].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_EXPECT_EMPTY: 
     "expect": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[34].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_EXPIRES_EMPTY: 
     "expires": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[35].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_FROM_EMPTY: 
     "from": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[36].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_HOST_EMPTY: 
     "host": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[37].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_IF_MATCH_EMPTY: 
     "if-match": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[38].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_IF_MODIFIED_SINCE_EMPTY: 
     "if-modified-since": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[39].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_IF_NONE_MATCH_EMPTY: 
     "if-none-match": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[40].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_IF_RANGE_EMPTY: 
     "if-range": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[41].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_IF_UNMODIFIED_SINCE_EMPTY: 
     "if-unmodified-since": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[42].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_LAST_MODIFIED_EMPTY: 
     "last-modified": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[43].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_LINK_EMPTY: 
     "link": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[44].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_LOCATION_EMPTY: 
     "location": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[45].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_MAX_FORWARDS_EMPTY: 
     "max-forwards": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[46].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_PROXY_AUTHENTICATE_EMPTY: 
     "proxy-authenticate": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[47].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_PROXY_AUTHORIZATION_EMPTY: 
     "proxy-authorization": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[48].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_RANGE_EMPTY: 
     "range": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[49].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_REFERER_EMPTY: 
     "referer": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[50].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_REFRESH_EMPTY: 
     "refresh": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[51].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_RETRY_AFTER_EMPTY: 
     "retry-after": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[52].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_SERVER_EMPTY: 
     "server": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[53].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_SET_COOKIE_EMPTY: 
     "set-cookie": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[54].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_STRICT_TRANSPORT_SECURITY_EMPTY: 
     "strict-transport-security": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[55].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_TRANSFER_ENCODING_EMPTY: 
     "transfer-encoding": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[56].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_USER_AGENT_EMPTY: 
     "user-agent": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[57].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_VARY_EMPTY: 
     "vary": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[58].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_VIA_EMPTY: 
     "via": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[59].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_WWW_AUTHENTICATE_EMPTY: 
     "www-authenticate": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[60].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_STATUS_0: 
     "grpc-status": "0" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[61].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_STATUS_1: 
     "grpc-status": "1" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[62].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_STATUS_2: 
     "grpc-status": "2" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[63].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_ENCODING_IDENTITY: 
     "grpc-encoding": "identity" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[64].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_ENCODING_GZIP: 
     "grpc-encoding": "gzip" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[65].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_ENCODING_DEFLATE: 
     "grpc-encoding": "deflate" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[66].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CONTENT_TYPE_APPLICATION_SLASH_GRPC: 
     "content-type": "application/grpc" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[67].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_SCHEME_GRPC: 
     ":scheme": "grpc" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[68].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_METHOD_PUT: 
     ":method": "PUT" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[69].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ACCEPT_ENCODING_EMPTY: 
     "accept-encoding": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[70].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CONTENT_ENCODING_IDENTITY: 
     "content-encoding": "identity" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[71].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_CONTENT_ENCODING_GZIP: 
     "content-encoding": "gzip" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[72].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_LB_COST_BIN_EMPTY: 
     "lb-cost-bin": "" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[73].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_ACCEPT_ENCODING_IDENTITY: 
     "grpc-accept-encoding": "identity" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[74].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_ACCEPT_ENCODING_DEFLATE: 
     "grpc-accept-encoding": "deflate" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[75].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_ACCEPT_ENCODING_IDENTITY_COMMA_DEFLATE: 
     "grpc-accept-encoding": "identity,deflate" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[76].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_ACCEPT_ENCODING_GZIP: 
     "grpc-accept-encoding": "gzip" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[77].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_ACCEPT_ENCODING_IDENTITY_COMMA_GZIP: 
     "grpc-accept-encoding": "identity,gzip" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[78].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_ACCEPT_ENCODING_DEFLATE_COMMA_GZIP: 
     "grpc-accept-encoding": "deflate,gzip" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[79].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_GRPC_ACCEPT_ENCODING_IDENTITY_COMMA_DEFLATE_COMMA_GZIP: 
     "grpc-accept-encoding": "identity,deflate,gzip" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[80].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ACCEPT_ENCODING_IDENTITY: 
     "accept-encoding": "identity" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[81].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ACCEPT_ENCODING_GZIP: 
     "accept-encoding": "gzip" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[82].data(),
        GRPC_MDELEM_STORAGE_STATIC),
    /* GRPC_MDELEM_ACCEPT_ENCODING_IDENTITY_COMMA_GZIP: 
     "accept-encoding": "identity,gzip" */
    GRPC_MAKE_MDELEM(
        &g_static_mdelem_table[83].data(),
        GRPC_MDELEM_STORAGE_STATIC)
    // clang-format on
};
}  // namespace grpc_core

uintptr_t grpc_static_mdelem_user_data[GRPC_STATIC_MDELEM_COUNT] = {
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 4, 4, 6, 6, 8, 8, 2, 4, 4};

static const int8_t elems_r[] = {
    18, 11, -8, 0,  3,  -75, -51, 0,  7,  -4,  0,   0,   0,   12,  -1,  -2,
    0,  0,  7,  6,  0,  0,   0,   0,  0,  0,   0,   0,   0,   0,   0,   0,
    0,  0,  0,  0,  0,  0,   0,   0,  0,  0,   0,   0,   0,   0,   0,   0,
    0,  0,  0,  0,  0,  0,   0,   0,  0,  -50, 0,   -55, -75, -76, -77, 0,
    30, 29, 28, 27, 26, 25,  24,  23, 22, 21,  28,  19,  18,  17,  16,  17,
    17, 16, 15, 14, 13, 12,  11,  10, 9,  8,   7,   6,   5,   4,   3,   2,
    1,  0,  -1, 0,  0,  0,   0,   0,  0,  0,   -11, 0};
static uint32_t elems_phash(uint32_t i) {
  i -= 46;
  uint32_t x = i % 108;
  uint32_t y = i / 108;
  uint32_t h = x;
  if (y < GPR_ARRAY_SIZE(elems_r)) {
    uint32_t delta = static_cast<uint32_t>(elems_r[y]);
    h += delta;
  }
  return h;
}

static const uint16_t elem_keys[] = {
    270,   271,   272,  273,  274,   275,  276,   1029,  1030,  1568,  1678,
    154,   155,   488,  489,  760,   919,  920,   46,    47,    1458,  1580,
    1690,  684,   685,  2008, 2118,  6628, 6738,  6848,  6958,  7068,  7178,
    7288,  7398,  7508, 7618, 7728,  7838, 7948,  1708,  8168,  8278,  8388,
    8498,  6518,  6298, 8608, 8058,  8718, 8828,  8938,  9048,  9158,  9268,
    9378,  9488,  9598, 9708, 9818,  9928, 10038, 10148, 10258, 10368, 10478,
    10588, 10698, 543,  1091, 10808, 214,  10918, 11578, 1096,  1097,  1098,
    1099,  981,   0,    0,    0,     1641, 1751,  0,     0,     0,     0,
    358,   1757,  0,    0,    0,     0,    1532};
static const uint8_t elem_idxs[] = {
    7,   8,   9,  10, 11,  12,  13,  75,  77, 25, 70,  1,   2,   5,   6,  61,
    66,  65,  3,  4,  30,  72,  82,  62,  63, 57, 37,  17,  18,  19,  20, 21,
    22,  23,  24, 26, 27,  28,  29,  31,  15, 33, 34,  35,  36,  16,  14, 38,
    32,  39,  40, 41, 42,  43,  44,  45,  46, 47, 48,  49,  50,  51,  52, 53,
    54,  55,  56, 58, 68,  74,  59,  69,  60, 73, 76,  78,  79,  80,  64, 255,
    255, 255, 71, 81, 255, 255, 255, 255, 0,  83, 255, 255, 255, 255, 67};

grpc_mdelem grpc_static_mdelem_for_static_strings(intptr_t a, intptr_t b) {
  if (a == -1 || b == -1) return GRPC_MDNULL;
  uint32_t k = static_cast<uint32_t>(a * 110 + b);
  uint32_t h = elems_phash(k);
  return h < GPR_ARRAY_SIZE(elem_keys) && elem_keys[h] == k &&
                 elem_idxs[h] != 255
             ? GRPC_MAKE_MDELEM(
                   &grpc_core::g_static_mdelem_table[elem_idxs[h]].data(),
                   GRPC_MDELEM_STORAGE_STATIC)
             : GRPC_MDNULL;
}

const uint8_t grpc_static_accept_encoding_metadata[8] = {0,  74, 75, 76,
                                                         77, 78, 79, 80};

const uint8_t grpc_static_accept_stream_encoding_metadata[4] = {0, 81, 82, 83};
