/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/type/v3/percent.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg_internal.h"
#include "envoy/type/v3/percent.upb.h"
#include "udpa/annotations/status.upb.h"
#include "udpa/annotations/versioning.upb.h"
#include "validate/validate.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout_field envoy_type_v3_Percent__fields[1] = {
  {1, UPB_SIZE(0, 0), 0, 0, 1, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_type_v3_Percent_msginit = {
  NULL,
  &envoy_type_v3_Percent__fields[0],
  UPB_SIZE(8, 8), 1, false, 1, 255,
};

static const upb_msglayout_field envoy_type_v3_FractionalPercent__fields[2] = {
  {1, UPB_SIZE(4, 4), 0, 0, 13, _UPB_MODE_SCALAR},
  {2, UPB_SIZE(0, 0), 0, 0, 14, _UPB_MODE_SCALAR},
};

const upb_msglayout envoy_type_v3_FractionalPercent_msginit = {
  NULL,
  &envoy_type_v3_FractionalPercent__fields[0],
  UPB_SIZE(8, 8), 2, false, 2, 255,
};

#include "upb/port_undef.inc"

